package eu.clarin.cmdi.mdservice.model;

/**
 * The main model class, carrying users_query
 * able to parse and transform the user-string
 * and execute the query, ie issue appropriate request on the MDRepository
 * getting the url-wording right from MDRepositoryProxy
 *   
 * At the moment just responsible for querying MDRepository
 * perhaps unify approach for other proxies - subtyping this Class  
 */

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.z3950.zing.cql.CQLNode;
import org.z3950.zing.cql.CQLParseException;
import org.z3950.zing.cql.CQLParser;

import eu.clarin.cmdi.mdservice.internal.Utils;
import eu.clarin.cmdi.mdservice.internal.MDTransformer;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;

public class Query {	
	
	public static Logger log = Logger.getLogger("Query");
	
	public static String COLLECTION = "collection";
	public static String COLUMNS = "columns";
	public static String MODEL = "model";
	public static String VALUES = "values";
	public static String RECORDSET = "search";
	public static String RECORD = "record";
	public static String SRUEXTERN = "sruextern";
	
	public static String PARSED = "parsed";
	public static String UNPARSED = "unparsed";
	public static String PARSEERROR = "parseerror";
	
	private String type;
	/**
	 * meant a target url/service, allowing querying different services on per query basis
	 * not used yet 
	 */
	private String target;	
	
	private URL targetRequest;
	private String syntax = "cql";  /* cql, cmdIndex, xpath */
	private String query_string;
	//private String full_query_string;
	private CQLNode query_cql;
	private String msg;
	/**
	 * temporary default;
	 */
	private String collection="";
	private String columns="Id,name";
	private String startItem = "1";
	private String maximumItems = "50";
	private String squery = null;
	private String options = null;
	private String sort;
	private int maxdepth=1;
	
	private Map sruMap = null;
	/**
	 * reference to the result-object
	 */
	private Result result = new Result(this);

	
	public Query(String queryString) {		
		this(queryString, MODEL);
	}

	/**	
	 * main constructor, with user's querystring and type of the query, and context-collection (at the moment just one) 
	 * @param queryString
	 * @param type
	 * @param collection
	 */

	public Query(String queryString, String type, String collection, String columns) {
		this.type =type;
		if (queryString == null)queryString ="";
		setFullQueryString(queryString);
		setCollection(collection);
		setColumns(columns);
	}
	
	/**	
	 * main constructor, with user's querystring and type of the query, and context-collection (at the moment just one) 
	 * @param queryString
	 * @param type
	 * @param collection
	 */

	public Query(String queryString, String type, String collection) {
		this( queryString, type, collection, "Id,name");
	}

	/**	
	 * another constructor, with user's querystring and type of the query;
	 * problematic not setting the collection!  
	 * @param queryString
	 * @param type
	 */

	public Query(String queryString, String type) {
		this(queryString, type, "");
	}

	public Boolean isStatus(String qstatus) {
		return (qstatus.equals(getStatus()));
	}
	
	public String getStatus() {
		if (type.equals(Query.RECORDSET) && query_cql== null && (query_string != "")) {
			return Query.PARSEERROR;
		} else {
			return Query.UNPARSED;
		}
		
	}
	
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getQueryString() {
		return query_string;
	}

	public void setFullQueryString(String queryString) {
		
		query_string=queryString;
		
		if (queryString.trim().length() == 0){
			queryString = null;
		}
		/*
		String squery = createsqueryString(squeryString);
		if (squery != null && queryString != null){
			full_query_string = "(" + squery + " ) and (" + queryString + ")";
		} else if (squery != null) {
			full_query_string = squery;
		} else if (queryString != null){
			full_query_string = queryString;
		} else {
			full_query_string = "";
		}
		*/
		log.debug("QUERY.FULLQUERYSTRING:" + query_string);
		if (type.equals(RECORDSET) && (query_string.length() > 0)) {	
			parse();
			//preprocess();
		}
			
	}
	/*
	public String getFullQueryString() {
		return full_query_string;
	}
*/
	
	public void setCollection(String collection) {
		if (collection!=null) {
			this.collection = collection;
		}
	}
	
	public String getCollection() {
		return collection;
	}
	
	public String getColumns() {
		return columns;
	}
	
	public void setColumns(String columns) {
		if (columns!=null) {
			this.columns = columns;
		}
	}

	public String getSQuery() {
		return this.squery;
	}
	
	public void setSQuery(String squery) {
		if (squery!=null) {
			this.squery = squery;
		}
	}
	
	public String getOptions() {
		return options;
	}

	public void setOptions(String options) {
		this.options = options;
	}

	public String getSort() {
		return sort;
	}

	public void setSort(String sort) {
		this.sort = sort;
	}

	public String getStartItem() {
		return startItem;
	}
	
	public void setStartItem(String startItem) {
		if (startItem!=null) {
			if (startItem.equals("")){
				// sets default startItem
				this.startItem = "1";
			}else {
				this.startItem = startItem;
			}
		}
	}
	
	public String getMaximumItems() {
		return maximumItems;
	}
	
	public void setMaximumItems(String maximumItems) {
		if (maximumItems!=null) {
			if (maximumItems.equals("")){
				this.maximumItems = "50";
			} else {
				this.maximumItems = maximumItems;	
			}
		}
	}
	public void setMaxdepth(int maxdepth) {
		if (maxdepth > 0) {
			this.maxdepth = maxdepth;
		}
	}

	public int getMaxdepth() {
		return maxdepth;
	}

	public void setResult(Result result) {
		this.result = result;
	}


	public Result getResult() {
		return result;
	}
	
	public String getMsg() {
		return msg;
	}

	public void setMsg(String msg) {
		this.msg = msg;
	}

	
	public Map getSruMap() {
		return sruMap;
	}

	public void setSruMap(Map sruMap) {
		this.sruMap = sruMap;
	}

	
	// apply rules form squery
	public String createsqueryString (String _squery){
		String sq = null;
		
		if (_squery !=null){
			sq = "* any " + _squery;
		}

		return sq;
	}
	public static String getSimpleQueryString(String querystring) {
		String[] arr_and = querystring.split(" and ");
		String simple_form = "";
		String simple_form_all = "";
		String rel = "";
		
		for( int i=0;i<arr_and.length;i++){
			arr_and[i] = arr_and[i].trim();
			String[] arr_or = arr_and[i].split(" or ");
			simple_form = "";
			for( int j=0;j<arr_or.length;j++){
				arr_or[j] = arr_or[j].trim();
				while (arr_or[j].substring(0,1).equals("(") ) {
					arr_or[j] = arr_or[j].substring(1,arr_or[j].length());
					arr_or[j] = arr_or[j].trim();
				}
				while ( arr_or[j].substring(arr_or[j].length()-1,arr_or[j].length()).equals(")")){
					arr_or[j] = arr_or[j].substring(0,arr_or[j].length()-1);
					arr_or[j] = arr_or[j].trim();
				}
				if (j > 0) { 
					rel = " or ";
				} else {
					rel = "";
				}
				simple_form = simple_form + rel + arr_or[j];
			}
			
			if (arr_or.length > 1){
				simple_form = "(" + simple_form + ") ";
			}
			if (i > 0) { 
				rel = " and  ";
			} else {
				rel = "";
			}
			simple_form_all = simple_form_all + rel + simple_form;
			
		}
		
		return simple_form_all;
	}
	/**
	 * construct the URL-Param 
	 * @return
	 * @throws MalformedURLException
	 * TODO url-encode (especially the query-param) ! 
	 */
	public String toURLParam() throws MalformedURLException {
		
		String targetRequest;
		
		if (type.equals(MODEL)) {
			targetRequest = fromCMDIndex2Xpath() + "&maxdepth=" + getMaxdepth()  ; /* + "&maxdepth=" + getMaxdepth() );  "&collection=" + getCollection() + */
		} 
		else if (type.equals(COLLECTION)) {
			targetRequest = getCollection() + "&maxdepth=" + getMaxdepth()  ; /* + "&maxdepth=" + getMaxdepth() );  "&collection=" + getCollection() + */

		} else if (type.equals(SRUEXTERN)) {
			targetRequest = "";//fromCMDIndex2Xpath() + "&maxdepth=" + getMaxdepth()  ; /* + "&maxdepth=" + getMaxdepth() );  "&collection=" + getCollection() + */
			
			Set set = sruMap.entrySet();
			Iterator i = set.iterator();
			
		    while(i.hasNext()){
			      Map.Entry me = (Map.Entry)i.next();
			      targetRequest = targetRequest + "&" + me.getKey() + "=" + me.getValue();
			}
					
		} else if (type.equals(VALUES)) {
			targetRequest = fromCMDIndex2Xpath() ; /* + "&maxdepth=" + getMaxdepth() );  "&collection=" + getCollection() + */
			
			if (startItem != null) {
				targetRequest = targetRequest +  "&startItem=" + getStartItem();
			}
			if (maximumItems != null) {
				targetRequest = targetRequest +  "&maxItems=" + getMaximumItems();
			}
			if (sort != null) {
				targetRequest = targetRequest +  "&sort=" + getSort();
			}
			
		} else if (type.equals(RECORD)) {
			//  2010-07-11 this is just a hack, because url-encoding the handle acted strange
			//String corrid = getQueryString().replace('_', '/');
			//  2010-12-28 needed for the lucene-index to recognize..
				//  probably not the best place to handle -> move to repository: sanitize-query? 
			String corrid = getQueryString().replace(":","\\:").toLowerCase();
			log.debug("Query.toURLParam.corrid=" + corrid);
			targetRequest = "//MdSelfLink[ft:query(.,'" + corrid + "')]";			 
		} else {
				if (query_cql == null){
					targetRequest =  "//*" + "&collection=" + getCollection();
				} else {
					targetRequest = toXPath() + "&collection=" + getCollection();	
				}
				if (startItem != null) {
					targetRequest = targetRequest +  "&startItem=" + getStartItem();
				}
				if (maximumItems != null) {
					targetRequest = targetRequest +  "&maxItems=" + getMaximumItems();
				}
				if ((options != null)) {
					targetRequest = targetRequest +  "&format=xml-" + getOptions();
				}
		}
	
		return targetRequest;
		
}
	
	/**
	 * tries to parse the full_query_string according the CQL-syntax
	 * if successful returns the root-node of the parse-tree
	 */
	public CQLNode parse() {
		 
	  try {
		CQLParser parser = new CQLParser();		
		// cannot accept '-' at the CQL beginning
		String local_full_query_string = query_string;
		local_full_query_string = local_full_query_string.replace("-", "%2D");

		query_cql = parser.parse(local_full_query_string);
	  }
		catch (CQLParseException e) {
			// TODO better handling of failed parse  
			setMsg("Query.ParseError-"+ e.getClass() + ": " + e.getMessage());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
        
		return query_cql;
	}
	
	/**
	 */
	public void  preprocess() {
		 
	  try {	
		log.debug(toXCQL());
			
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document doc = builder.parse(new org.xml.sax.InputSource(new java.io.StringReader(toXCQL())));
			
		//log.debug("document created");
		SearchClauses searchclauses = new SearchClauses(doc);
		//do the processing
		searchclauses.process();
		String xml_s = searchclauses.toXML();
		
		
	  }
		catch (Exception e) {
			e.printStackTrace();
		}
		return;
	}
	
	
	/**
	 * provides a xml-version of the query (if parsed successfully)
	 * according to the XCQL-schema defined in the SRU/CQL standard	
	 * @return
	 */
	public String toXCQL() {	
		 return query_cql.toXCQL(0);
	}

	/**
	 * provides a xpath version of the query, based on the XCQL-version,
	 * applying a stylesheet on the XCQL-version  
	 * @return XPath-query
	 * @throws IOException 
	 */
	public String toXPath() {	
		MDTransformer transformer = new MDTransformer();
		transformer.setParams(MDTransformer.createParamsMap("XCQL2XPATH"));
		try {
			return transformer.transformXML(toXCQL());
		} catch (NoStylesheetException e) {
			setMsg("Query.toXPath-"+ e.getClass() + ": " + e.getMessage());
			return "";
		}
	}

	/**
	 * if the query is just a path-like structure
	 * "transforms" the cmdIndex-format to appropriate XPath, means replace('.', '/') ;)
	 * and resolves back possible profile-prefix
	 * @return
	 */
	public String fromCMDIndex2Xpath() {
		String xpath = "";
		if (query_string.contains(":")) {
			int delim_index = query_string.indexOf(":");
			String profile_name = query_string.substring(0,delim_index );
			Termset res = (Termset) Termset.getTermset(profile_name);			  
			if (res!=null) {
				profile_name= res.getAttr("name");
			} 
			if (profile_name.equals("")) {
				xpath=query_string.substring(delim_index); 
			} else {
				xpath = profile_name + "//" + query_string.substring(delim_index+1);
			}
				
		} else {
			xpath = query_string; 
		}
				
		 return xpath.replace(".", "//"); 
	}

	
}