package eu.clarin.cmdi.mdservice.proxy;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import org.apache.log4j.Logger;

import eu.clarin.cmdi.mdservice.internal.CQLParseException;
import eu.clarin.cmdi.mdservice.model.Diagnostic;
import eu.clarin.cmdi.mdservice.model.Query;
import eu.clarin.cmdi.mdservice.model.WorkspaceProfile;

/**
 * Proxy to the Metadata Repository.  
 * Responds to requests (collections, model, recordset) 
 * by dispatching the requests to appropriate internal methods and filling the inputStream with the result
 * 
 * It is possible to switch target-repository dynamically  (on every request). I.e. the client explicitly selects one of the repositories.
 *  
 * @author vronk
 *
 */
public class MDRepoProxy extends RepoProxy {

	private static final long serialVersionUID = 1L;
	private static Logger log = Logger.getLogger("MDRepoProxy");
	
	private String proxy_key = "mdrepository";
	
	/**
	 * Override of the basic method provided by the super-class 
	 */	
	@Override	
	public String getBaseURI() {		
		String uri = WorkspaceProfile.getRepositoryPath(getSourceAction().getRepository());		
		return uri;
	}
	
	@Override
	public String getProxyKey() {
		return proxy_key;
	}
	
	/**
	 * A mapping between the actionkeys in the request and the operation-parameter expected by the MDRepository 
	 */
	private final static HashMap<String,String> urls = new HashMap<String,String>();
	 static
	 {	 urls.put("collection", "?operation=getCollections&collection=");
	  	 urls.put("model", "?operation=queryModel&q=");
	  	 urls.put("values", "?operation=scanIndex&q=");
	 	 urls.put("search", "?operation=searchRetrieve&query=");
	 	 urls.put("record", "?operation=searchRetrieve&query=");
	 } 
 
	/**
	 * uses base_url + url_pattern (parametrized by actionkey) to form a url
	 * @return the full target-request URL to be sent to the target repository
	 * @throws CQLParseException 
	 * @throws Exception 
	 */
	 @Override
	public URL getTargetRequest() throws IOException, CQLParseException {
        
		//TODO squery removed
		//Query query = new Query(getParam("squery"), getParam("query"),getSourceAction().getActionkey());
		Query query = new Query(getParam("query"),getSourceAction().getActionkey());
	
		//log.debug("MDRPx.getQ:" + getSquery() + " and (" +  getQ() + ")");
		log.debug("MDRP.getQuery:" + getParam("query"));
		log.debug("MDRP.getActionkey:" + getSourceAction().getActionkey());		

	// check if the query could get parsed
		if (query.isStatus(Query.PARSEERROR)) {
			log.debug("MDRP.query.PARSEERROR:" + query.getMsg());
			throw new CQLParseException("MDRP.query.PARSEERROR:" + query.getMsg());
			//getSourceAction().getDiagnostics().Add(Diagnostic.SYSTEM_ERROR, "MDRP.query.PARSEERROR:" + query.getMsg());
			//TODO exception ??
			//return null;
		} else {
			query.setMaxdepth(Integer.valueOf(getParam("maxdepth")));		
			query.setCollection(getParam("collections"));
			query.setColumns(getParam("columns"));
			query.setMaximumItems(getParam("maximumRecords"));
			query.setStartItem(getParam("startRecord"));
			query.setOptions(getParam("options"));
			query.setSort(getParam("sort"));
		
			URL targetURL = null;
			targetURL =new URL( getBaseURL(), urls.get(getSourceAction().getActionkey()) + query.toURLParam());
		
			log.debug("MDRPA.targetURL:" + targetURL);
	        return targetURL;
		}
		 
	}
	
}